/*
  Copyright 2021 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.friend.util

import java.util.*

/**
 * @suppress
 */
class FriendOldComparator : Comparator<NameComparable> {

    override fun compare(o1: NameComparable?, o2: NameComparable?): Int {
        return compare(o1?.phoneticNameForSorting, o2?.phoneticNameForSorting)
    }

    fun compare(s1: String?, s2: String?): Int {
        if (s1.isNullOrBlank() && s2.isNullOrBlank()) {
            return 0
        }
        if (s1.isNullOrBlank()) {
            return 1
        }
        if (s2.isNullOrBlank()) {
            return -1
        }

        val leftC = PhonemeUtils.getFirstPhoneme(s1)
        val rightC = PhonemeUtils.getFirstPhoneme(s2)

        var result = PhonemeUtils.startsWith(leftC) - PhonemeUtils.startsWith(rightC)
        if (result == 0) {
            result = leftC - rightC
            if (result == 0) {
                return s1.compareTo(s2)
            }
        }
        return result
    }

}