/*
  Copyright 2021 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.user.network

import android.content.Context
import com.kakao.sdk.auth.AuthApiClient
import com.kakao.sdk.auth.RxAuthApiClient
import com.kakao.sdk.auth.model.OAuthToken
import com.kakao.sdk.auth.rx
import com.kakao.sdk.common.KakaoSdk

import com.kakao.sdk.common.model.ApiError
import com.kakao.sdk.common.model.ApiErrorCause
import com.kakao.sdk.common.model.ApplicationContextInfo
import com.kakao.sdk.common.model.ContextInfo
import com.kakao.sdk.common.util.SdkLog
import com.kakao.sdk.partner.auth.RxAgeAuthApiClient
import com.kakao.sdk.partner.auth.verifyAge
import io.reactivex.*

import io.reactivex.internal.operators.flowable.FlowableOnBackpressureBufferStrategy
import io.reactivex.schedulers.Schedulers
import org.reactivestreams.Publisher
/**
 * @suppress
 */
class RxUserOperations (private val authApiClient: RxAgeAuthApiClient = RxAgeAuthApiClient.instance,
                        private val contextInfo: ApplicationContextInfo = KakaoSdk.applicationContextInfo) {


    fun handleCompletableError(context: Context = contextInfo.applicationContext): CompletableTransformer =
            CompletableTransformer {
            it.retryWhen {
                 ageVerificationRequired(context, it)
            }
        }


    @JvmSynthetic
    internal fun ageVerificationRequired(context: Context = contextInfo.applicationContext, throwableFlowable: Flowable<Throwable>): Publisher<Unit> =
            throwableFlowable.flatMap {
                if (it !is ApiError || it.reason != ApiErrorCause.RequiredAgeVerification) {
                    throw it
                }
                return@flatMap authApiClient.verifyAgeWithAuthentication(context).toFlowable().observeOn(Schedulers.io())
            }

    companion object {
        @JvmStatic
        val instance by lazy {
            RxUserOperations()
        }
    }
}